/*
 * $Id: sfic.h 107 2006-07-12 21:18:41Z ncopa $
 */
#ifndef _SFIC_H
#define _SFIC_H	1


 /* Just a silly construct to contain "global" variables          */
typedef struct
{
  char *writedbName;		/* the database we write to                     */
  char *readdbName;		/* the database we read from on compare         */
  heap_t *fileList;		/* the actual list of filespecs                 */
  heap_t *outputList;		/* the list of stuff to output 			*/
  int operation;		/* the operation (add,delete,compare,list)      */
  int verbosity;		/* how verbose should we be                     */
  unsigned mask;		/* the list of atributes we check (pinugsamch)  */
  unsigned ignore;		/* the file types we ignore			*/
  char *separator;		/* field separator for output			*/
  char *maskstr;		/* pointer to the original mask			*/
  unsigned flags;		/* filesystem flags				*/
  TDB_CONTEXT *writetdb;	/* the (new) database context                   */
  TDB_CONTEXT *readtdb;		/* the (old,existing) database context          */
} sfic_t;

/* the file info record */
typedef struct {
	struct stat statbuf;	/* stat info 					*/
	int    hashbits;	/* how many bits in the hash			*/
	unsigned char   hash[64];	/* save at least 512 bits for the hash		*/
	} file_info_t;

#define STAT_TYPE      1
#define STAT_PERM      2
#define STAT_INODE     4
#define STAT_NLINK     8
#define STAT_UID       16
#define STAT_GID       32
#define STAT_SIZE      64
#define STAT_ATIME     132
#define STAT_MTIME     256
#define STAT_CTIME     512
#define STAT_HASH      1024


#define FSPEC_NONE		0
#define FSPEC_RECURSE		1
#define FSPEC_NOMOUNTPOINTS 	2
#define FSPEC_DOHASH		4
#define FSPEC_NOTRIVIAL		8

/* sfic.c */
void die_with_error(const char *s, ...);
void *xmalloc(size_t size);
void *xrealloc(void *buf, size_t size);
void xreport(int level, const char *s, ...);
char charFromMode(mode_t mode);
void statToString(char *string, char *maskstr, file_info_t *data, char *filename);
void formatOutputListEntry (char *buf, size_t size, int verbosity, char *str1, char *str2, char *str3, char *name);
void convertMaskToString(int mask, char *maskstr);
int convertMaskFromString(char *str);
unsigned convert_ignore_from_string(char *str);
unsigned int lstatDelta(file_info_t *a, file_info_t *b, unsigned int checkmask);
int sha256FileByName(char *filename, unsigned char *sum);
void printOutputListHeap(heap_t *list);
void addHeap(char *key, void *data, size_t size, heap_t *heap);
void addDirToHeap(char *dirspec, dev_t parentdev, unsigned flags, heap_t *fileList);
void addHeapFromFile(char *source, heap_t *fileList);
char *get_next_file_spec(file_info_t *info, sfic_t *sf);
int addHeapEntriesToDB(sfic_t *sf);
int remove_heap_entries_from_db(sfic_t *sf);
void listEntriesFromDB(sfic_t *sf);
int compareTdbContents(sfic_t *sf);
void printUsage(sfic_t *globals);
void initializeSficStruct(sfic_t *sf);
int parseCommandLine(int argc, char *argv[], sfic_t *globals);
int main(int argc, char *argv[]);


#endif /* _SFIC_H */
